@@ -25,8 +25,6 @@ import os
 import re
 from xml.etree import ElementTree
 
-import six
-
 from . import core
 from . import mixin
 from .datatypes import TYPE_TO_PYTHON
@@ -34,178 +32,9 @@ from .constants import SECONDARY_LOOKUP_FIELDS, FIELD_HINTS, CORE_REST_OBJECTS
 from .utils import pythonize_class_name, pythonize_attribute_name, full_class_name
 
 
-FILE_HEADER = '''
-# Copyright 2011-2015 Biomedical Imaging Group Rotterdam, Departments of
-# Medical Informatics and Radiology, Erasmus MC, Rotterdam, The Netherlands
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-from __future__ import absolute_import
-from __future__ import unicode_literals
-import os
-import tempfile  # Needed by generated code
-from gzip import GzipFile  # Needed by generated code
-from tarfile import TarFile  # Needed by generated code
-from zipfile import ZipFile  # Needed by generated code
-from six import BytesIO  # Needed by generated code
-
-from xnat import search, mixin
-from xnat.core import XNATObject, XNATNestedObject, XNATSubObject, XNATListing, XNATSimpleListing, XNATSubListing, caching
-from xnat.utils import mixedproperty, RequestsFileLike
-
-try:
-    PYDICOM_LOADED = True
-    import pydicom
-except ImportError:
-    PYDICOM_LOADED = False
-
-SESSION = None
-
-
-def current_session():
-    return SESSION
-
-
-# These mixins are to set the xnat_session automatically in all created classes
-class XNATObjectMixin(XNATObject):
-    @mixedproperty
-    def xnat_session(self):
-        return current_session()
-
-    @classmethod
-    def query(cls, *constraints):
-        query = search.Query(cls, cls.xnat_session)
-
-        # Add in constraints immediatly
-        if len(constraints) > 0:
-            query = query.filter(*constraints)
-
-        return query
-
-
-class XNATNestedObjectMixin(XNATNestedObject):
-    @mixedproperty
-    def xnat_session(self):
-        return current_session()
-
-
-class XNATSubObjectMixin(XNATSubObject):
-    @mixedproperty
-    def xnat_session(self):
-        return current_session()
-
-
-class FileData(XNATObjectMixin):
-    SECONDARY_LOOKUP_FIELD = "{file_secondary_lookup}"
-    _XSI_TYPE = 'xnat:fileData'
-
-    def __init__(self, uri=None, xnat_session=None, id_=None, datafields=None, parent=None, fieldname=None, overwrites=None, path=None):
-        super(FileData, self).__init__(uri=uri,
-                                       xnat_session=xnat_session,
-                                       id_=id_,
-                                       datafields=datafields,
-                                       parent=parent,
-                                       fieldname=fieldname,
-                                       overwrites=overwrites)
-
-        # Save id because we need it if cache gets wiped
-        self._id = id_
-        
-        if path is not None:
-            self._path = path
-            
-    @property
-    def id(self):
-        return self._id
-
-    @property
-    def path(self):
-        return self._path
-
-    def delete(self):
-        self.xnat_session.delete(self.uri)
-
-    def download(self, *args, **kwargs):
-        self.xnat_session.download(self.uri, *args, **kwargs)
-
-    def download_stream(self, *args, **kwargs):
-        self.xnat_session.download_stream(self.uri, *args, **kwargs)
-        
-    def open(self):
-        uri = self.xnat_session._format_uri(self.uri)
-        request = self.xnat_session.interface.get(uri, stream=True)
-        return RequestsFileLike(request)
-        
-    @property
-    @caching
-    def fulldata(self):
-        listing_uri = self.uri[:-len(self.id)-1]
-        data = self.xnat_session.get_json(listing_uri)
-        data = data['ResultSet']['Result']
-        item = next(x for x in data if x['Name'] == self.id)
-        return item
-        
-    @property
-    def data(self):
-        return self.fulldata
-        
-    @property
-    def cat_id(self):
-        return self.get('cat_ID', str)
-        
-    @property
-    def collection(self):
-        return self.get('collection', str)
-        
-    @property
-    def digest(self):
-        return self.get('digest', str)
-        
-    @property
-    def file_content(self):
-        return self.get('file_content', str)
-        
-    @property
-    def file_format(self):
-        return self.get('file_format', str)
-        
-    @property
-    def file_tags(self):
-        return self.get('file_tags', str)
-        
-    @property
-    def file_size(self):
-        return self.get('Size', str)
-
-    @property
-    @caching
-    def size(self):
-        response = self.xnat_session.head(self.uri, allow_redirects=True)
-        return response.headers['Content-Length']
-
-
-# Empty class lookup to place all new lookup values
-XNAT_CLASS_LOOKUP = {{
-    "xnat:fileData": FileData,
-}}
-
-
-# The following code represents the data structure of the XNAT server
-# It is automatically generated using
-{schemas}
-
-
-'''
+XNATPY_NAMESPACE = 'https://gitlab.com/radiology/infrastructure/xnatpy'
+with open(os.path.join(os.path.dirname(__file__), 'header.py')) as fin:
+    FILE_HEADER = fin.read()
 
 # TODO: Add more fields to FileData from [Name, Size, URI, cat_ID, collection, file_content, file_format, tile_tags]?
 # TODO: Add display identifiers support (DONE?)
@@ -216,18 +45,16 @@ XNAT_CLASS_LOOKUP = {{
 # <xs:documentation>An individual person involved in experimental research</xs:documentation>
 # </xs:annotation>
 # <xs:sequence>
-# TODO: Add XPATHs for setting SubObjects (SEMI-DONE)
 # TODO: Make Listings without key and with numeric index possible (inProgress)
 # TODO: Fix scan parameters https://groups.google.com/forum/#!topic/xnat_discussion/GBZoamC2ZmY
-# TODO: Check the nesting weirdness in petScanDataParametersFramesFrames (DONE?)
-# TODO: Figure out the object/subobject/semi-subobject mess.
 # TODO: Move all system function to use a __ prefix
 
 
 class ClassPrototype(object):
-    def __init__(self, parser, name, logger, field_name=None, parent_class=None, simple=False):
+    def __init__(self, parser, name, namespace, logger, field_name=None, parent_class=None, simple=False):
         self.parser = parser
-        self.name = name  # This is the XSI type
+        self._name = name  # This is the XSI type
+        self.namespace = namespace
         self.logger = logger
 
         self.field_name = field_name
@@ -244,7 +71,23 @@ class ClassPrototype(object):
         self.source_schema = self.parser.current_schema
 
     def __repr__(self):
-        return "<ClassPrototype {}>".format(self.name)
+        return f"<ClassPrototype {self.name}>"
+
+    @property
+    def name(self):
+        if self.namespace:
+            return '{}:{}'.format(self.parser.namespace_prefixes[self.namespace], self._name)
+        else:
+            return self._name
+
+    @name.setter
+    def name(self, value):
+        if ':' in value:
+            prefix, name = value.split(":", 2)
+            self.namespace = self.parser.namespaces[prefix]
+            self._name = name
+        else:
+            self._name = value
 
     def root_base_class(self, topxsd=False):
         base = self.base_class
@@ -527,10 +370,15 @@ class BaseClassWriter(BaseWriter):
             header += "    _DISPLAY_IDENTIFIER = '{}'\n".format(self.display_identifier)
 
         if 'fields' in self.attributes:
-            header += "    _HAS_FIELDS = True\n"
+            attribute = self.attributes['fields']
+            element_class = attribute.element_class
+            # Make sure it's only proper fields and not a subobject called fields (THIS DOES HAPPEN!)
+            if element_class.class_type == 'Simple':
+                header += "    _HAS_FIELDS = True\n"
+                header += "    custom_variables = search.SearchFieldMap('{name}')\n".format(name=self.name)
 
         if self.parent_class is not None:
-            header += "    #_PARENT_CLASS = {}\n".format(self.python_parent_class)
+            header += "    _PARENT_CLASS = {}\n".format(self.python_parent_class)
             header += "    _FIELD_NAME = '{}'\n".format(self.field_name)
         elif self.name in FIELD_HINTS:
             header += "    _CONTAINED_IN = '{}'\n".format(FIELD_HINTS[self.name])
@@ -792,7 +640,7 @@ class PropertyWriter(AttributeWriter):
     """    @mixedproperty
     def {clean_name}(cls):{docstring}
         # 0 Automatically generated Property, type: {type_}
-        return search.SearchField(cls, "{name}")
+        return search.SearchField(cls, "{name}", "{type_}")
 
     @{clean_name}.getter
     def {clean_name}(self):{docstring}
@@ -921,7 +769,7 @@ class ListingPropertyWriter(AttributeWriter):
         if self.element_class is not None:
             element_class = self.element_class
             property_base += element_class.writer.create_listing(secondary_lookup=secondary_lookup, field_name=field_name)
-        elif isinstance(self.type, six.string_types) and not self.type.startswith('xs:'):
+        elif isinstance(self.type, str) and not self.type.startswith('xs:'):
             element_class = self.parser.class_list[self.type]
             property_base += element_class.writer.create_listing(secondary_lookup=secondary_lookup, field_name=field_name)
         else:
@@ -934,12 +782,11 @@ class ListingPropertyWriter(AttributeWriter):
 class SchemaParser(object):
     def __init__(self, logger, debug=False):
         # Manage XML namespaces
-        self.namespaces = {}
-        self.namespace_prefixes = {}
+        self.namespaces = {'xnatpy': XNATPY_NAMESPACE}
+        self.namespace_prefixes = {XNATPY_NAMESPACE: 'xnatpy'}
         self.target_namespace = ''
         self.current_schema = None
 
-        self.class_list = collections.OrderedDict()
         self.class_list = collections.OrderedDict()
         self.unknown_tags = set()
         self.new_class_stack = [None]
@@ -1035,6 +882,8 @@ class SchemaParser(object):
         tries = 0
         yielded_anything = True
         while len(visited) < len(self.class_list) and yielded_anything and tries < 25:
+            if self.debug:
+                self.logger.info(f"============ Iteration {tries} ============")
             yielded_anything = False
             for key, value in self.class_list.items():
                 if key in visited:
@@ -1043,14 +892,14 @@ class SchemaParser(object):
                 base = value.base_class
                 if base is not None and not base.startswith('xs:') and base not in visited:
                     if self.debug:
-                        self.logger.debug("Wait with processing {} because base {} is not yet processed".format(
+                        self.logger.info("Wait with processing {} because base {} is not yet processed".format(
                             value.name, base
                         ))
                     continue
 
                 if value.parent_class is not None and value.parent_class not in visited:
                     if self.debug:
-                        self.logger.debug("Wait with processing {} because parent {} is not yet processed".format(
+                        self.logger.info("Wait with processing {} because parent {} is not yet processed".format(
                             value.name, value.parent_class
                         ))
                     continue
@@ -1063,14 +912,13 @@ class SchemaParser(object):
             tries += 1
 
         expected = len(self.class_list) + nr_previsited  # We started with two "visited" classes
-        if self.debug:  # and len(visited) < len(self.class_list):
+        if len(visited) < expected:
             missed = sorted(set(self.class_list) - visited)
-            if self.debug:
-                self.logger.debug('Visited: {}, expected: {}'.format(len(visited), expected))
-                self.logger.debug('Visited: {}'.format(visited))
-                self.logger.info('Missed: {}'.format(missed))
-                self.logger.info('Missed base class: {}'.format([self.class_list[x].base_class for x in missed]))
-                self.logger.debug('Spent {} iterations'.format(tries))
+            self.logger.debug('Visited: {}, expected: {}'.format(len(visited), expected))
+            self.logger.debug('Visited: {}'.format(visited))
+            self.logger.info('Missed: {}'.format(missed))
+            self.logger.info('Missed base class: {}'.format([self.class_list[x].base_class for x in missed]))
+            self.logger.debug('Spent {} iterations'.format(tries))
 
     @contextlib.contextmanager
     def _descend(self, new_class=None, new_property=None, property_prefix=None):
@@ -1148,15 +996,20 @@ class SchemaParser(object):
         field_name = None
 
         if name is None:
-            name = 'xnatpy:' + self._current_class.name.split(":", 1)[-1] + self._current_property.name.capitalize()
+            name = self._current_class.name.split(":", 1)[-1] + self._current_property.name.capitalize()
+            namespace = XNATPY_NAMESPACE
             parent_class = self._current_class.name
             field_name = self._current_property.name
         else:
-            if ':' not in name:
-                name = self.target_namespace_prefix + name
+            if ':' in name:
+                namespace_prefix, name = name.split(':', 2)
+                namespace = self.namespaces[namespace_prefix]
+            else:
+                namespace = self.target_namespace
 
         new_class = ClassPrototype(self,
                                    name=name,
+                                   namespace=namespace,
                                    logger=self.logger,
                                    field_name=field_name,
                                    parent_class=parent_class)
@@ -1165,7 +1018,7 @@ class SchemaParser(object):
             self._current_property.element_class = new_class
             new_class.parent_property = self._current_property
 
-        self.class_list[name] = new_class
+        self.class_list['{}:{}'.format(namespace, name)] = new_class
 
         # Descend
         with self._descend(new_class=new_class):
@@ -1219,14 +1072,16 @@ class SchemaParser(object):
             name = new_base[3:].capitalize()
             if self._current_property is not None:
                 new_prop = self._current_property.name
-                new_base = 'xnatpy:{}{}'.format(new_prop, name)
+                new_base = '{}{}'.format(new_prop, name)
             else:
                 new_prop = self._current_class.name
                 new_base = self._current_class.name + name
-                new_base = 'xnatpy:{}'.format(new_base.split(':', 1)[1])
+                new_base = new_base.split(':', 1)[1]
+            namespace = XNATPY_NAMESPACE
 
             new_base_class = ClassPrototype(self,
                                             name=new_base,
+                                            namespace=namespace,
                                             logger=self.logger,
                                             simple=True)
 
@@ -1234,7 +1089,10 @@ class SchemaParser(object):
                                                                      name=new_prop,
                                                                      logger=self.logger,
                                                                      type=previous_base)
-            self.class_list[new_base] = new_base_class
+            self.class_list['{}:{}'.format(namespace, new_base)] = new_base_class
+
+            # For lookup later we do need the xnatpy namespace prefix
+            new_base = 'xnatpy:{}'.format(new_base)
 
         self._current_class.base_class = new_base
         self._parse_children(element)
@@ -1268,10 +1126,6 @@ class SchemaParser(object):
 
     def _parse_schema(self, element):
         self.target_namespace = element.get("targetNamespace", '')
-        if self.target_namespace != '':
-            self.target_namespace_prefix = self.namespace_prefixes[self.target_namespace] + ':'
-        else:
-            self.target_namespace_prefix = ''
 
         for child in list(element):
             if child.tag in [
@@ -1305,11 +1159,15 @@ class SchemaParser(object):
             return
 
         # This is the top-level of schema and a sort of typedef
-        if ':' not in name:
-            name = self.target_namespace_prefix + name
+        if ':' in name:
+            namespace_prefix, name = name.split(':', 2)
+            namespace = self.namespaces[namespace_prefix]
+        else:
+            namespace = self.target_namespace
 
         new_class = ClassPrototype(self,
                                    name=name,
+                                   namespace=namespace,
                                    logger=self.logger,
                                    simple=True)
 
@@ -1319,7 +1177,7 @@ class SchemaParser(object):
                                           type=None)
         new_class.attributes["value"] = new_property
 
-        self.class_list[name] = new_class
+        self.class_list['{}:{}'.format(namespace, name)] = new_class
 
         # Descend
         with self._descend(new_class=new_class, new_property=new_property):
@@ -1376,6 +1234,7 @@ class SchemaParser(object):
 
     def prune_tree(self):
         to_remove = set()
+        renamed = {}
 
         for cls in self.class_list.values():
             for property_key, prop in cls.attributes.items():
@@ -1395,6 +1254,7 @@ class SchemaParser(object):
                         # The new element_class has to be updated to take the place of the old element_class
                         new_element_class = element_property.element_class
                         if new_element_class is not None:
+                            renamed[new_element_class.name] = element_class.name
                             new_element_class.name = element_class.name
                             new_element_class.field_name = '{}/{}'.format(element_class.field_name,
                                                                           new_element_class.field_name)
@@ -1406,15 +1266,29 @@ class SchemaParser(object):
                                     element_class.name)
                                 )
                             to_remove.add(element_class.name)
-
                         cls.attributes[property_key] = element_property
                     elif self.debug:
                         self.logger.debug("Ignoring non-listing...")
                         self.logger.debug("Element class: {}".format(element_class.__dict__))
 
         for key in to_remove:
+            if self.debug:
+                self.logger.info(f'Removing pruned class {key}')
             del self.class_list[key]
 
+        for cls in self.class_list.values():
+            if cls.parent_class in renamed:
+                new_name = renamed[cls.parent_class]
+                if self.debug:
+                    self.logger.info(f'Fixing parent name after rename in pruning from {cls.parent_class} to {new_name}')
+                cls.parent_class = new_name
+
+        for old_name, new_name in renamed.items():
+            if self.debug:
+                self.logger.info(f'Re-inserting renamed class {old_name} in class list under {new_name}')
+            item = self.class_list.pop(old_name)
+            self.class_list[new_name] = item
+
         for cls in self.class_list.values():
             for property_key, prop in cls.attributes.items():
                 # Only consider simplifying listings
@@ -1448,14 +1322,20 @@ class SchemaParser(object):
             self.logger.debug('namespaces: {}'.format(self.namespaces))
             self.logger.debug('namespace prefixes: {}'.format(self.namespace_prefixes))
 
+        # Update class list to include namespaces
+        new_class_list = collections.OrderedDict()
+        for value in self.class_list.values():
+            new_class_list[value.name] = value
+        self.class_list = new_class_list
+
         self.logger.info('Pruning data structure')
         before = set(self.class_list.keys())
         if self.debug:
-            self.logger.debug('Classed before prune: {}'.format(before))
+            self.logger.debug('Classes before prune: {}'.format(before))
         self.prune_tree()
         after = set(self.class_list.keys())
         if self.debug:
-            self.logger.debug('Classed after prune: {}'.format(after))
+            self.logger.debug('Classes after prune: {}'.format(after))
             self.logger.info('Classes removed by pruning: {}'.format(sorted(before - after)))
 
         self.logger.info('Writing result')
